==> jPod-5.3-sources.jar: de/intarsys/pdf/cos/COSObjectWalkerDeep.java <==
/*
 * Copyright (c) 2007, intarsys consulting GmbH
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * - Neither the name of intarsys nor the names of its contributors may be used
 *   to endorse or promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package de.intarsys.pdf.cos;

import java.util.HashSet;
import java.util.Set;

/**
 * Adapter implementation for visiting a COS object structure, including
 * indirect references.
 * <p>
 * Every object in the potential cyclic data structure is visited exactly once.
 * 
 */
public class COSObjectWalkerDeep extends COSObjectWalkerShallow {
	private Set visited;

	private boolean swap;

	public COSObjectWalkerDeep() {
		this(true);
	}

	public COSObjectWalkerDeep(boolean swap) {
		visited = new HashSet();
		this.swap = swap;
	}

	public Set getVisited() {
		return visited;
	}

	protected void handleException(RuntimeException e)
			throws COSVisitorException {
		throw new COSVisitorException(e);
	}

	@Override
	public Object visitFromIndirectObject(COSIndirectObject io)
			throws COSVisitorException {
		if (getVisited().contains(io)) {
			return null;
		}
		getVisited().add(io);
		if (swap || !io.isSwapped()) {
			try {
				io.dereference().accept(this);
			} catch (RuntimeException e) {
				handleException(e);
			}
		}
		return null;
	}
}

==> jPod-5.5.1-sources.jar: de/intarsys/pdf/cos/COSObjectWalkerDeep.java <==
/*
 * Copyright (c) 2007, intarsys consulting GmbH
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * - Neither the name of intarsys nor the names of its contributors may be used
 *   to endorse or promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package de.intarsys.pdf.cos;

import java.util.HashSet;
import java.util.Set;

/**
 * Adapter implementation for visiting a COS object structure, including
 * indirect references.
 * <p>
 * Every object in the potential cyclic data structure is visited exactly once.
 * 
 */
public class COSObjectWalkerDeep extends COSObjectWalkerShallow {
	private Set visited;

	final private boolean forceSwap;

	final private boolean swapAlways;

	public COSObjectWalkerDeep() {
		this(true);
	}

	public COSObjectWalkerDeep(boolean forceSwap) {
		visited = new HashSet();
		this.forceSwap = forceSwap;
		this.swapAlways = forceSwap;
	}

	public COSObjectWalkerDeep(boolean forceSwap, boolean swapAlways) {
		visited = new HashSet();
		this.forceSwap = forceSwap;
		this.swapAlways = swapAlways;
	}

	public Set getVisited() {
		return visited;
	}

	protected void handleException(RuntimeException e)
			throws COSVisitorException {
		throw new COSVisitorException(e);
	}

	@Override
	public Object visitFromIndirectObject(COSIndirectObject io)
			throws COSVisitorException {
		if (getVisited().contains(io)) {
			return null;
		}
		getVisited().add(io);
		// descend if
		// - we should always swap
		// - we should swap and object was previously read
		// - object is swapped in
		if (swapAlways || forceSwap && !io.isInitialState() || !io.isSwapped()) {
			try {
				io.dereference().accept(this);
			} catch (RuntimeException e) {
				handleException(e);
			}
		}
		return null;
	}
}
